@@ -583,6 +583,9 @@ mpc_data<T> create_contact_slip_condition(
                           num_slaves_recv3.data(), disp3.data(),
                           dolfinx::MPI::mpi_type<U>(), neighborhood_comms[0]);
 
+  int err0 = MPI_Comm_free(&neighborhood_comms[0]);
+  dolfinx::MPI::check_error(comm, err0);
+
   // Compute off-process contributions
   mpc_data<T> remote_data;
   {
@@ -735,6 +738,9 @@ mpc_data<T> create_contact_slip_condition(
   /// Wait for all communication to finish
   MPI_Waitall(4, requests.data(), status.data());
 
+  int err1 = MPI_Comm_free(&neighborhood_comms[1]);
+  dolfinx::MPI::check_error(comm, err1);
+
   // Move the masters, coeffs and owners from the input adjacency list
   // to one where each node corresponds to an entry in slave_indices_remote
   std::vector<std::int32_t> offproc_offsets(slave_indices_remote.size() + 1, 0);
@@ -893,11 +899,13 @@ mpc_data<T> create_contact_slip_condition(
 /// @param[in] master_marker Tag for the other interface
 /// @param[in] eps2 The tolerance for the squared distance to be considered a
 /// collision
+/// @param[in] allow_missing_masters If true, the function will not throw an error if
+/// a degree of freedom in the closure of the master entities does not have a corresponding set of slave degrees of freedom.
 template <typename T, std::floating_point U>
 mpc_data<T> create_contact_inelastic_condition(
     const dolfinx::fem::FunctionSpace<U>& V,
     dolfinx::mesh::MeshTags<std::int32_t> meshtags, std::int32_t slave_marker,
-    std::int32_t master_marker, const U eps2 = 1e-20)
+    std::int32_t master_marker, const U eps2 = 1e-20, bool allow_missing_masters = false)
 {
   dolfinx::common::Timer timer("~MPC: Inelastic condition");
 
@@ -953,10 +961,6 @@ mpc_data<T> create_contact_inelastic_condition(
   std::array<MPI_Comm, 2> neighborhood_comms
       = create_neighborhood_comms(comm, meshtags, has_slave, master_marker);
 
-  // Create communicator local_blocks -> ghost_block
-  MPI_Comm slave_to_ghost
-      = create_owner_to_ghost_comm(local_blocks, ghost_blocks, imap);
-
   /// Compute which rank (relative to neighbourhood) to send each ghost to
   std::span<const int> ghost_owners = imap->owners();
 
@@ -1083,7 +1087,7 @@ mpc_data<T> create_contact_inelastic_condition(
   if (int mpi_size = dolfinx::MPI::size(comm); mpi_size == 1)
   {
 
-    if (!blocks_wo_local_collision.empty())
+    if ((!blocks_wo_local_collision.empty()) and (!allow_missing_masters))
     {
       throw std::runtime_error(
           "No masters found on contact surface (when executed in serial). "
@@ -1100,7 +1104,40 @@ mpc_data<T> create_contact_inelastic_condition(
     // slaves)
     std::vector<std::int32_t> slaves;
     slaves.reserve(block_size * local_blocks.size());
-    std::ranges::for_each(
+    // If we allow missing masters, we need to skip those slaves from the MPC
+    if ((!blocks_wo_local_collision.empty()))
+    {
+      int counter = 0;
+      std::ranges::for_each(
+        local_blocks,
+        [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
+         &local_coeffs, &offsets_out, &slaves, &counter, &collision_to_local](const std::int32_t block)
+        {
+          if (std::find(collision_to_local.begin(), collision_to_local.end(), counter++) != collision_to_local.end())
+          {
+            std::cout << "Skipping block " << block << " without local collision." << counter << std::endl;
+            return; // Skip blocks without local collision
+          }
+          else
+          {
+
+          for (std::int32_t j = 0; j < block_size; ++j)
+          {
+            const std::int32_t slave = block * block_size + j;
+            slaves.push_back(slave);
+            masters_out.insert(masters_out.end(), local_masters[slave].begin(),
+                               local_masters[slave].end());
+            coeffs_out.insert(coeffs_out.end(), local_coeffs[slave].begin(),
+                              local_coeffs[slave].end());
+            offsets_out.push_back((std::int32_t)masters_out.size());
+          }
+        }
+        });
+
+    }
+    else
+    {
+      std::ranges::for_each(
         local_blocks,
         [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
          &local_coeffs, &offsets_out, &slaves](const std::int32_t block)
@@ -1116,6 +1153,7 @@ mpc_data<T> create_contact_inelastic_condition(
             offsets_out.push_back((std::int32_t)masters_out.size());
           }
         });
+      }
     std::vector<std::int32_t> owners(masters_out.size());
     std::ranges::fill(owners, 0);
     mpc.slaves = slaves;
@@ -1171,6 +1209,9 @@ mpc_data<T> create_contact_inelastic_condition(
                           num_block_coordinates.data(), coordinate_disp.data(),
                           dolfinx::MPI::mpi_type<U>(), neighborhood_comms[0]);
 
+  int err0 = MPI_Comm_free(&neighborhood_comms[0]);
+  dolfinx::MPI::check_error(comm, err0);
+
   // Vector for processes with slaves, mapping slaves with
   // collision on this process
   std::vector<std::vector<std::int64_t>> collision_slaves(indegree);
@@ -1394,6 +1435,9 @@ mpc_data<T> create_contact_inelastic_condition(
       recv_num_found_blocks.data(), inc_block_disp.data(),
       dolfinx::MPI::mpi_type<std::int32_t>(), neighborhood_comms[1]);
 
+  int err1 = MPI_Comm_free(&neighborhood_comms[1]);
+  dolfinx::MPI::check_error(comm, err1);
+
   // Iterate through the processors
   for (std::size_t i = 0; i < src_ranks_rev.size(); ++i)
   {
@@ -1459,7 +1503,7 @@ mpc_data<T> create_contact_inelastic_condition(
   {
     for (int d = 0; d < block_size; d++)
     {
-      if (local_masters[block * block_size + d].empty())
+      if ((local_masters[block * block_size + d].empty()) and (!allow_missing_masters))
       {
         throw std::runtime_error(
             "No masters found on contact surface for slave. Please run in "
@@ -1476,6 +1520,8 @@ mpc_data<T> create_contact_inelastic_condition(
       ghost_slaves[i * tdim + j] = ghost_blocks[i] * block_size + j;
 
   // Compute source and dest ranks of communicator
+  MPI_Comm slave_to_ghost
+      = create_owner_to_ghost_comm(local_blocks, ghost_blocks, imap);
   auto neighbour_ranks = dolfinx_mpc::compute_neighborhood(slave_to_ghost);
   const std::vector<int>& src_ranks_ghost = neighbour_ranks.first;
   const std::vector<int>& dest_ranks_ghost = neighbour_ranks.second;
@@ -1640,6 +1686,9 @@ mpc_data<T> create_contact_inelastic_condition(
       disp_recv_ghost_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
       slave_to_ghost);
 
+  int err3 = MPI_Comm_free(&slave_to_ghost);
+  dolfinx::MPI::check_error(comm, err3);
+
   // Accumulate offsets of masters from different processors
   std::vector<std::int32_t> ghost_offsets = {0};
   for (std::size_t i = 0; i < src_ranks_ghost.size(); ++i)
@@ -1674,7 +1723,10 @@ mpc_data<T> create_contact_inelastic_condition(
       {
         for (std::int32_t j = 0; j < tdim; ++j)
         {
+          // Add the slave if it has masters
           const std::int32_t slave = block * block_size + j;
+          if (local_masters[slave].empty())
+            continue;
           slaves.push_back(slave);
           masters.insert(masters.end(), local_masters[slave].begin(),
                          local_masters[slave].end());
@@ -1695,8 +1747,13 @@ mpc_data<T> create_contact_inelastic_condition(
   coeffs_out.reserve(num_local_masters + num_ghost_masters);
   owners_out.reserve(num_local_masters + num_ghost_masters);
   offsets.reserve(num_local_masters + num_ghost_masters + 1);
+  std::vector<std::int32_t> local_ghosts
+      = map_dofs_global_to_local<U>(V, in_ghost_slaves);
   for (std::int32_t i = 0; i < num_ghost_slaves; i++)
   {
+    if (ghost_offsets[i+1]  - ghost_offsets[i] == 0)
+      continue; // Skip if no masters for ghost slave
+    slaves.push_back(local_ghosts[i]);
     for (std::int32_t j = ghost_offsets[i]; j < ghost_offsets[i + 1]; j++)
     {
       masters.push_back(in_ghost_masters[j]);
@@ -1705,12 +1762,6 @@ mpc_data<T> create_contact_inelastic_condition(
     }
     offsets.push_back((std::int32_t)masters.size());
   }
-
-  // Map ghosts to local data and copy into slave array
-  std::vector<std::int32_t> local_ghosts
-      = map_dofs_global_to_local<U>(V, in_ghost_slaves);
-  slaves.resize(num_loc_slaves + num_ghost_slaves);
-  std::ranges::copy(local_ghosts, slaves.begin() + num_loc_slaves);
   mpc.slaves = slaves;
   mpc.masters = masters;
   mpc.offsets = offsets;
