@@ -11,10 +11,13 @@
 #include <dolfinx/fem/DirichletBC.h>
 #include <dolfinx/fem/Form.h>
 #include <functional>
+using mdspan2_t = MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+    const std::int32_t,
+    MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>;
 
 namespace dolfinx_mpc
 {
-template <typename T>
+template <typename T, std::floating_point U>
 class MultiPointConstraint;
 
 /// Given a local element vector, move all slave contributions to the global
@@ -22,7 +25,7 @@ class MultiPointConstraint;
 /// @param [in, out] b The global (local to process) vector
 /// @param [in, out] b_local The local element vector
 /// @param [in] b_local_copy Copy of the local element vector
-/// @param [in] cell_blocks Dofmap for the blocks in the cell
+/// @param [in] dofs The cell dofs (blocked)
 /// @param [in] num_dofs The number of degrees of freedom in the local vector
 /// @param [in] bs The element block size
 /// @param [in] is_slave Vector indicating if a dof is a slave
@@ -32,10 +35,9 @@ class MultiPointConstraint;
 template <typename T>
 void modify_mpc_vec(
     const std::span<T>& b, const std::span<T>& b_local,
-    const std::span<T>& b_local_copy,
-    const std::span<const std::int32_t>& cell_blocks, const int num_dofs,
-    const int bs, const std::vector<std::int8_t>& is_slave,
-    const std::span<const std::int32_t>& slaves,
+    const std::span<T>& b_local_copy, std::span<const std::int32_t> dofs,
+    const int num_dofs, const int bs, std::span<const std::int8_t> is_slave,
+    std::span<const std::int32_t> slaves,
     const std::shared_ptr<const dolfinx::graph::AdjacencyList<std::int32_t>>&
         masters,
     const std::shared_ptr<const dolfinx::graph::AdjacencyList<T>>& coeffs)
@@ -44,7 +46,7 @@ void modify_mpc_vec(
   // NOTE: Should this be moved into the MPC constructor?
   // Get local index of slaves in cell
   std::vector<std::int32_t> local_index
-      = compute_local_slave_index(slaves, num_dofs, bs, cell_blocks, is_slave);
+      = compute_local_slave_index(slaves, num_dofs, bs, dofs, is_slave);
 
   // Move contribution from each slave to corresponding master dof
   for (std::size_t i = 0; i < local_index.size(); i++)
@@ -59,7 +61,8 @@ void modify_mpc_vec(
         b[masters_i[j]] += coeffs_i[j] * b_local_copy[local_index[i]];
       else
         // Use Hermitian transpose for type std::complex<double>
-        b[masters_i[j]] += std::conj(coeffs_i[j]) * b_local_copy[local_index[i]];
+        b[masters_i[j]]
+            += std::conj(coeffs_i[j]) * b_local_copy[local_index[i]];
       b_local[local_index[i]] = 0;
     }
   }
@@ -72,8 +75,8 @@ void modify_mpc_vec(
 /// @param[in] mpc The multi-point constraint
 void assemble_vector(
     std::span<double> b, const dolfinx::fem::Form<double>& L,
-    const std::shared_ptr<const dolfinx_mpc::MultiPointConstraint<double>>&
-        mpc);
+    const std::shared_ptr<
+        const dolfinx_mpc::MultiPointConstraint<double, double>>& mpc);
 
 /// Assemble a linear form into a vector
 /// @param[in] b The vector to be assembled. It will not be zeroed before
@@ -84,6 +87,29 @@ void assemble_vector(
     std::span<std::complex<double>> b,
     const dolfinx::fem::Form<std::complex<double>>& L,
     const std::shared_ptr<
-        const dolfinx_mpc::MultiPointConstraint<std::complex<double>>>& mpc);
+        const dolfinx_mpc::MultiPointConstraint<std::complex<double>, double>>&
+        mpc);
+
+/// Assemble a linear form into a vector
+/// @param[in] b The vector to be assembled. It will not be zeroed before
+/// assembly.
+/// @param[in] L The linear forms to assemble into b
+/// @param[in] mpc The multi-point constraint
+void assemble_vector(
+    std::span<float> b, const dolfinx::fem::Form<float>& L,
+    const std::shared_ptr<
+        const dolfinx_mpc::MultiPointConstraint<float, float>>& mpc);
+
+/// Assemble a linear form into a vector
+/// @param[in] b The vector to be assembled. It will not be zeroed before
+/// assembly.
+/// @param[in] L The linear forms to assemble into b
+/// @param[in] mpc The multi-point constraint
+void assemble_vector(
+    std::span<std::complex<float>> b,
+    const dolfinx::fem::Form<std::complex<float>>& L,
+    const std::shared_ptr<
+        const dolfinx_mpc::MultiPointConstraint<std::complex<float>, float>>&
+        mpc);
 
 } // namespace dolfinx_mpc
