@@ -55,7 +55,12 @@ def demo_stacked_cubes(
 
     # Read in mesh
     if gmsh:
-        mesh, mt = gmsh_3D_stacked(celltype, theta, res)
+        # Offset one box for partial contact
+        if noslip:
+            offset = 0.235
+        else:
+            offset = 0.0
+        mesh, mt = gmsh_3D_stacked(celltype, theta, res, offset=offset)
         tdim = mesh.topology.dim
         fdim = tdim - 1
         mesh.topology.create_connectivity(tdim, tdim)
@@ -119,12 +124,12 @@ def demo_stacked_cubes(
     # + inner(g, v) * ds
     bilinear_form = fem.form(a)
     linear_form = fem.form(rhs)
-
     mpc = MultiPointConstraint(V)
-    tol = float(5e2 * np.finfo(default_scalar_type).resolution)
+    tol = float(5e3 * np.finfo(default_scalar_type).resolution)
     if noslip:
         with Timer("~~Contact: Create non-elastic constraint"):
-            mpc.create_contact_inelastic_condition(mt, 4, 9, eps2=tol)
+            mpc.create_contact_inelastic_condition(mt, 4, 9, eps2=tol, allow_missing_masters=True)
+
     else:
         with Timer("~Contact: Create contact constraint"):
             nh = create_normal_approximation(V, mt, 4)
@@ -132,7 +137,7 @@ def demo_stacked_cubes(
 
     with Timer("~~Contact: Add data and finialize MPC"):
         mpc.finalize()
-
+    num_slaves = MPI.COMM_WORLD.allreduce(mpc.num_local_slaves, op=MPI.SUM)
     # Create null-space
     null_space = rigid_motions_nullspace(mpc.function_space)
     num_dofs = V.dofmap.index_map.size_global * V.dofmap.index_map_bs
@@ -144,7 +149,6 @@ def demo_stacked_cubes(
     apply_lifting(b, [bilinear_form], [bcs], mpc)
     b.ghostUpdate(addv=PETSc.InsertMode.ADD_VALUES, mode=PETSc.ScatterMode.REVERSE)  # type: ignore
     fem.petsc.set_bc(b, bcs)
-
     # Solve Linear problem
     opts = PETSc.Options()  # type: ignore
     opts["ksp_rtol"] = 1.0e-8
@@ -156,8 +160,8 @@ def demo_stacked_cubes(
     opts["mg_levels_pc_type"] = "jacobi"
     opts["mg_levels_esteig_ksp_type"] = "cg"
     opts["matptap_via"] = "scalable"
-    # opts["pc_gamg_square_graph"] = 2
-    # opts["pc_gamg_threshold"] = 1e-2
+    opts["pc_gamg_square_graph"] = 2
+    opts["pc_gamg_threshold"] = 1e-2
     # opts["help"] = None # List all available options
     # opts["ksp_view"] = None # List progress of solver
 
@@ -169,11 +173,6 @@ def demo_stacked_cubes(
 
     u_h = fem.Function(mpc.function_space)
     with Timer("~~Contact: Solve"):
-        # Temporary fix while:
-        # https://gitlab.com/petsc/petsc/-/issues/1339
-        # gets sorted
-        A.convert("baij", A)
-        A.convert("aij", A)
         solver.solve(b, u_h.x.petsc_vec)
         u_h.x.scatter_forward()
 
