@@ -3,16 +3,20 @@
 # This file is part of DOLFINX_MPC
 #
 # SPDX-License-Identifier:    MIT
+from __future__ import annotations
 
 import typing
 
+import dolfinx
 import dolfinx.fem as fem
 import numpy as np
-import numpy.typing
-from petsc4py import PETSc
+from dolfinx import default_scalar_type
 
 
-def close_to(point: np.typing.NDArray[np.float64]):
+def close_to(
+    point: np.typing.NDArray[typing.Union[np.float64, np.float32]],
+    atol=1000 * np.finfo(dolfinx.default_real_type).resolution,
+):
     """
     Convenience function for locating a point [x,y,z]
     within an array x [[x0,...,xN],[y0,...,yN], [z0,...,zN]].
@@ -20,14 +24,16 @@ def close_to(point: np.typing.NDArray[np.float64]):
     Args:
         point: The point should be padded to 3D
     """
-    return lambda x: np.isclose(x, point).all(axis=0)
+    return lambda x: np.isclose(x, point, atol=atol).all(axis=0)
 
 
 @typing.no_type_check
-def create_dictionary_constraint(V: fem.FunctionSpace, slave_master_dict:
-                                 typing.Dict[bytes, typing.Dict[bytes, float]],
-                                 subspace_slave: typing.Optional[int] = None,
-                                 subspace_master: typing.Optional[int] = None):
+def create_dictionary_constraint(
+    V: fem.functionspace,
+    slave_master_dict: typing.Dict[bytes, typing.Dict[bytes, float]],
+    subspace_slave: typing.Optional[int] = None,
+    subspace_master: typing.Optional[int] = None,
+):
     """
     Returns a multi point constraint for a given function space
     and dictionary constraint.
@@ -46,11 +52,10 @@ def create_dictionary_constraint(V: fem.FunctionSpace, slave_master_dict:
         .. highlight:: python
         .. code-block:: python
 
-            {np.array([d0, d1], dtype=numpy.float64).tobytes():
-                {numpy.array([e0, e1], dtype=numpy.float64).tobytes(): alpha,
-                numpy.array([f0, f1], dtype=numpy.float64).tobytes(): beta}}
+            {np.array([d0, d1], dtype=mesh.geometry.x.dtype).tobytes():
+                {numpy.array([e0, e1], dtype=mesh.geometry.x.dtype).tobytes(): alpha,
+                numpy.array([f0, f1], dtype=mesh.geometry.x.dtype).tobytes(): beta}}
     """
-    dfloat = np.float64
     comm = V.mesh.comm
     bs = V.dofmap.index_map_bs
     local_size = V.dofmap.index_map.size_local * bs
@@ -60,6 +65,7 @@ def create_dictionary_constraint(V: fem.FunctionSpace, slave_master_dict:
     non_local_entities = {}
     slaves_local = {}
     slaves_ghost = {}
+    dfloat = V.mesh.geometry.x.dtype
     slave_point_nd = np.zeros((3, 1), dtype=dfloat)
     for i, slave_point in enumerate(slave_master_dict.keys()):
         num_masters = len(list(slave_master_dict[slave_point].keys()))
@@ -69,63 +75,71 @@ def create_dictionary_constraint(V: fem.FunctionSpace, slave_master_dict:
         sp = np.frombuffer(slave_point, dtype=dfloat)
         for j, coord in enumerate(sp):
             slave_point_nd[j] = coord
-        slave_point_nd[len(sp):] = 0
+        slave_point_nd[len(sp) :] = 0
 
         if subspace_slave is None:
             slave_dofs = fem.locate_dofs_geometrical(V, close_to(slave_point_nd))
         else:
             Vsub = V.sub(subspace_slave).collapse()[0]
-            slave_dofs = fem.locate_dofs_geometrical(
-                (V.sub(subspace_slave), Vsub), close_to(slave_point_nd))[0]
+            slave_dofs = fem.locate_dofs_geometrical((V.sub(subspace_slave), Vsub), close_to(slave_point_nd))[0]
         if len(slave_dofs) == 1:
             # Decide if slave is ghost or not
             if slave_dofs[0] < local_size:
                 slaves_local[i] = slave_dofs[0]
-                owned_entities[i] = {"masters": np.full(num_masters, -1, dtype=np.int64),
-                                     "coeffs": np.full(num_masters, -1, dtype=np.float64),
-                                     "owners": np.full(num_masters, -1, dtype=np.int32),
-                                     "master_count": 0, "local_index": []}
+                owned_entities[i] = {
+                    "masters": np.full(num_masters, -1, dtype=np.int64),
+                    "coeffs": np.full(num_masters, -1, dtype=dolfinx.default_scalar_type),
+                    "owners": np.full(num_masters, -1, dtype=np.int32),
+                    "master_count": 0,
+                    "local_index": [],
+                }
                 slave_status = 1
             else:
                 slaves_ghost[i] = slave_dofs[0]
-                ghosted_entities[i] = {"masters": np.full(num_masters, -1, dtype=np.int64),
-                                       "coeffs": np.full(num_masters, -1, dtype=np.float64),
-                                       "owners": np.full(num_masters, -1, dtype=np.int32),
-                                       "master_count": 0, "local_index": []}
+                ghosted_entities[i] = {
+                    "masters": np.full(num_masters, -1, dtype=np.int64),
+                    "coeffs": np.full(num_masters, -1, dtype=dolfinx.default_scalar_type),
+                    "owners": np.full(num_masters, -1, dtype=np.int32),
+                    "master_count": 0,
+                    "local_index": [],
+                }
                 slave_status = 0
         elif len(slave_dofs) > 1:
-            raise RuntimeError("Multiple slaves found at same point. "
-                               + "You should use sub-space locators.")
+            raise RuntimeError("Multiple slaves found at same point. " + "You should use sub-space locators.")
         # Wrap as list to ensure order later
         master_points = list(slave_master_dict[slave_point].keys())
         master_points_nd = np.zeros((3, len(master_points)), dtype=dfloat)
-        for (j, master_point) in enumerate(master_points):
+        for j, master_point in enumerate(master_points):
             # Wrap bytes as numpy array
             for k, coord in enumerate(np.frombuffer(master_point, dtype=dfloat)):
                 master_points_nd[k, j] = coord
             if subspace_master is None:
-                master_dofs = fem.locate_dofs_geometrical(V, close_to(master_points_nd[:, j:j + 1]))
+                master_dofs = fem.locate_dofs_geometrical(V, close_to(master_points_nd[:, j : j + 1]))
             else:
                 Vsub = V.sub(subspace_master).collapse()[0]
-                master_dofs = fem.locate_dofs_geometrical((V.sub(subspace_master), Vsub),
-                                                          close_to(master_points_nd[:, j:j + 1]))[0]
+                master_dofs = fem.locate_dofs_geometrical(
+                    (V.sub(subspace_master), Vsub), close_to(master_points_nd[:, j : j + 1])
+                )[0]
 
             # Only add masters owned by this processor
             master_dofs = master_dofs[master_dofs < local_size]
             if len(master_dofs) == 1:
                 master_block = master_dofs[0] // bs
-                master_rem = master_dofs % bs
-                glob_master = index_map.local_to_global([master_block])[0]
+                master_rem = master_dofs[0] % bs
+                glob_master = index_map.local_to_global(np.asarray([master_block], dtype=np.int32))[0]
                 if slave_status == -1:
                     if i in non_local_entities.keys():
                         non_local_entities[i]["masters"].append(glob_master * bs + master_rem)
                         non_local_entities[i]["coeffs"].append(slave_master_dict[slave_point][master_point])
-                        non_local_entities[i]["owners"].append(comm.rank),
+                        (non_local_entities[i]["owners"].append(comm.rank),)
                         non_local_entities[i]["local_index"].append(j)
                     else:
-                        non_local_entities[i] = {"masters": [glob_master * bs + master_rem],
-                                                 "coeffs": [slave_master_dict[slave_point][master_point]],
-                                                 "owners": [comm.rank], "local_index": [j]}
+                        non_local_entities[i] = {
+                            "masters": [glob_master * bs + master_rem],
+                            "coeffs": [slave_master_dict[slave_point][master_point]],
+                            "owners": [comm.rank],
+                            "local_index": [j],
+                        }
                 elif slave_status == 0:
                     ghosted_entities[i]["masters"][j] = glob_master * bs + master_rem
                     ghosted_entities[i]["owners"][j] = comm.rank
@@ -209,6 +223,10 @@ def create_dictionary_constraint(V: fem.FunctionSpace, slave_master_dict:
         owners.extend(ghosted_slaves[slave_index]["owners"])  # type: ignore
         coeffs.extend(ghosted_slaves[slave_index]["coeffs"])  # type: ignore
         offsets.append(len(masters))
-    return (np.asarray(slaves, dtype=np.int32), np.asarray(masters, dtype=np.int64),
-            np.asarray(coeffs, dtype=PETSc.ScalarType), np.asarray(owners, dtype=np.int32),
-            np.asarray(offsets, dtype=np.int32))
+    return (
+        np.asarray(slaves, dtype=np.int32),
+        np.asarray(masters, dtype=np.int64),
+        np.asarray(coeffs, dtype=default_scalar_type),
+        np.asarray(owners, dtype=np.int32),
+        np.asarray(offsets, dtype=np.int32),
+    )
